#include <avr/io.h>
#define F_CPU 16000000UL // frekvence hodinovho signlu (nutn pro delay.h)
#include <util/delay.h>
#include <stdlib.h> 
#include <stdio.h>

#define pole 0x08
#define vlevo 0x03

char had[pole] = {0b11111111, 0b11111111, 0b11111111, 0b11000111,
               0b11111111, 0b11111111, 0b11111111, 0b11111111};
char jablko[pole] = {0b11111111, 0b11111111, 0b11111111, 0b11111111,
               0b11111111, 0b11111111, 0b11111111, 0b11111111};
char hlava[2] = {3, 5}; //vychozi souradnice hlavy, 3 znaci 4 radek v hadovi a 5 znaci 6 bit v danem radku 
char jablkosour[2] = {0, 0};
char souradnice[64][2]; 
int size = 3; //vychozi velikost hada


void jablicko (void) //vygeneruji nahodne jablko
{
  int n, i;
  int konec = 1;

  while (konec) //budu cyklit dokud se jablicko nevygeneruje mimo hada
  {
    n = rand() % 64; //celociselny vyraz od 0 do 63
    jablkosour[0] = n / 8;
    jablkosour[1] = n % 8;
	konec = 0;
	for (i = 0; i < size ; i++) //kontroluji, jestli nedoslo ke kolizi
    {
      if ((souradnice[i][0] == jablkosour[0]) & (souradnice[i][1] == jablkosour[1]))
	  {
        konec = 1;
	  }
    }
  }
  for (i = 0; i < pole; i++) //jablkosour se promitnou do jablka[pole]
  {
    if (jablkosour[0] == i) //hledam radek, na kterem je jablko
	{
	  jablko[i] &= ~(1 << jablkosour[1]); //souradnice udavaji misto jednoho clanku v hadovi
	}
  }
}



void nulovani_jablka (void)
{
  int i;

  for (i = 0; i < pole; i++)
  {
    jablko[i] = 0xff;
  }
}


void nulovani_hada (void)
{
  int i;

  for (i = 0; i < pole; i++)
  {
    had[i] = 0xff;
  }
}


void kolize (void) //uvedu hada do vychozi pozice jako v deklaraci promennych
{
  int i;

  size = 3; 
  for (i = 0; i < size; i++)
  {
    souradnice[i][0] = 3;
	souradnice[i][1] = 3 + i;
  }
  hlava[0] = 3;
  hlava[1] = 5;
  nulovani_jablka (); //predtim si uklidime
  jablicko (); //pro noveho hada nova potrava
  _delay_ms (500); //stalo se nestesti, had se zakousl, cekame, nez se prisune novy
}


char dej_hada (int radek)
{
  return had[radek];
}



char dej_jablko (int radek)
{
  return jablko[radek];
}


char dej_hlavu (int pozice)
{
  return hlava[pozice];
}


void inicializace_souradnic_hada (void) //vychozi pozice hada v poli had
{
  int i;

  for (i = 0; i < size; i++)
  {
    souradnice[i][0] = 3;
	souradnice[i][1] = 3 + i;
  }
  for (i = size; i < 64; i++)
  {
    souradnice[i][0] = 0;
	souradnice[i][1] = 0;
  }
}



int zmena (int pohyb)
{
  int i, j;
  int sezrano = 0; //promenna pro kontrolu, zdali had sezral jablicko
  int error = 0; //zatim kolize nenastala

  switch (pohyb)
  {
  case 0: //vpravo
    hlava[1] --;
	if (hlava[1] < 0)
      hlava[1] = 7;
	break;

  case 1: //dolu
    hlava[0] --;
	if (hlava[0] < 0)
      hlava[0] = 7;
	break;

  case 2: //nahoru
    hlava[0] ++;
	if (hlava[0] > 7)
      hlava[0] = 0;
	break;

  case 3: //vlevo
	hlava[1] ++;
	if (hlava[1] > 7) //kontroluji, jestli se had nedostal mimo matici
      hlava[1] = 0;
    break;
  }

    for (i = 1; i < size ; i++) //kontroluji, jestli nedoslo ke kolizi
  {
    if ((souradnice[i][0] == hlava[0]) & (souradnice[i][1] == hlava[1]))
	{
	  kolize(); //nastala kolize, hada vratim do vychozi pozice
	  error = 1;
	  pohyb = vlevo; //nastala kolize, tak menim pohyb na vychozi
	  break; //vyskocim z cyklu
	}
  }

  if ((hlava[0] == jablkosour[0]) & (hlava[1] == jablkosour[1])) //sezral had jablko?
  {
    sezrano = 1;
	size ++;
  }

  if (sezrano)
  {
    souradnice[size - 1][0] = hlava[0]; //pridam novou souradnici
    souradnice[size - 1][1] = hlava[1];
	nulovani_jablka (); //jablko je sezrane, pryc s nim
	jablicko (); //stara potrava je sezrana, pridam novou
  }
  else
  {
    for (i = 0; i < (size - 1); i++)
      for (j = 0; j < 2; j++)
	  {
	    souradnice[i][j] = souradnice[i + 1][j]; //nejstarsi souradnice zmizi
	  }
    souradnice[size - 1][0] = hlava[0]; //pridam novou souradnici
    souradnice[size - 1][1] = hlava[1];
  }

  nulovani_hada (); //zbavim se stare pozice

  for (i = 0; i < size; i++) //souradnice se promitnou do hada
    for (j = 0; j < pole; j++)
	{
      if (souradnice[i][0] == j) //hledam radek, na kterem je clanek hada
	  {
	    had[j] &= ~(1 << souradnice[i][1]); //souradnice udavaji misto jednoho clanku v hadovi
	  }
	}
  return error;
}


